// filebrws.prg

// Program sequence using tbrowse() to browse a file. Assumes
// all lines are terminated with CR / LF

#include "fileio.ch"
#include "inkey.ch"

#define CR    chr(13)
#define LF    chr(10)
#define CRLF  CR + LF

// File's current position
#define FTELL(handle)  fseek(handle, 0, FS_RELATIVE)

STATIC nHandle    // The handle for the file we open
STATIC cLine      // The current line
STATIC nFileSize  // The size of the file

// Pass file name as a parameter

FUNCTION test(fname)

LOCAL to, tbco, tbco1, n, lExitRequested := .F., nKey

    CLEAR SCREEN
    @ 0, 0 TO 24, 79

    IF lFileInit(fname)
        to := tbrowsenew(1, 1, 23, 78)
        n := 1
        tbco := tbColumnNew(, {|| substr(cCurLine(), n) })

        tbco:width := 78
        to:addcolumn(tbco)

        to:goTopBlock    := {| | GetFirst() }
        to:goBottomBlock := {| | GetLast()  }
        to:skipBlock     := {|n| skipper(n) }

        DO WHILE !lExitRequested
          DO WHILE nextkey() == 0 .AND. !to:stabilize()
          ENDDO

          nKey := inkey(0)
          DO CASE
            CASE nKey == K_LEFT
              IF n > 1
                n--
                to:refreshall()
              ENDIF

            CASE nKey == K_RIGHT
              IF n < len(cCurLine())
                n++
                to:refreshall()
              ENDIF

            CASE nKey == K_HOME
              n := 1
              to:refreshall()

            CASE nKey == K_END
              n := max(1, len(cCurline()) - 78 + 1)
              to:refreshAll()

            CASE nKey == K_ESC
              lExitRequested := .T.

            #define TAB_SIZE 10

            CASE nKey == K_TAB
              IF n <= len(cCurline()) - TAB_SIZE
                n += TAB_SIZE
                to:refreshall()
              ENDIF

            CASE nKey == K_SH_TAB
              n = MAX(1, n - TAB_SIZE)
              to:refreshall()

            OTHERWISE
              IF stdmeth(nKey, to)
                //  Handled as standard key
              ELSE
                // Otherwise ignore
              ENDIF
          ENDCASE
        ENDDO

        CLEAR SCREEN
    ENDIF

RETURN NIL


// Return the current line

FUNCTION cCurLine

RETURN cLine


#define MAX_LINE_LEN 256

// Initialization routine - open file, set nFileSize, and get first line

FUNCTION lFileInit(nFname)

    IF (nHandle := fopen(nFname)) > 0
        fseek(nHandle, 0, FS_END)
        nFileSize := FTELL(nHandle)
        GetFirst()
    ENDIF

RETURN nHandle != 0


// Read first line and place in the variable cline

FUNCTION GetFirst

    fseek(nHandle, 0, FS_SET)
    lfreadln(nHandle, @cline)
    fseek(nHandle, 0, FS_SET)

RETURN NIL


// Read the next line from the file into "cBuffer". The routine
// returns .T. if read, .F. otherwise (indicating EOF)
// The caller must pass cBuffer by reference

FUNCTION lFreadln(nHandle, cBuffer)

LOCAL nEol, nRead, nSaveFpos

    cBuffer := space(MAX_LINE_LEN)

    // First save current file pointer
    nSaveFpos := FTELL(nHandle)
    nRead = fread(nHandle, @cBuffer, MAX_LINE_LEN)

    IF (nEol := at(CRLF, substr(cBuffer, 1, nRead))) = 0
        // Line overflow or eof
        // Cline has the line we need
    ELSE
        // Copy up to eol
        cBuffer := substr(cBuffer, 1, nEol - 1)
        // Position file pointer to next line
        fseek(nHandle, nSaveFpos + nEol + 1, FS_SET)
    ENDIF

RETURN nRead != 0


// Read last line in file and place in variable cline

FUNCTION GetLast

    fseek(nHandle, 0, FS_END)
    lGoPrevLn()

RETURN NIL


// Skip n lines in the file. n can be positive or negative

FUNCTION skipper(n)

LOCAL nSkipped := 0

    IF n > 0
        DO WHILE nSkipped != n .AND. lGoNextLn()
            nSkipped++
        ENDDO
    ELSE
        DO WHILE nSkipped != n .AND. lGoPrevLn()
            nSkipped--
        ENDDO
    ENDIF

RETURN nSkipped


// Attempt to move to the next line in the file
// Return .T. if successful, .F. otherwise

FUNCTION lGoNextLn()

LOCAL nSavePos := FTELL(nHandle), ;
      cBuff := "", lMoved, nNewPos

    fseek(nHandle, len(cLine) + 2, FS_RELATIVE)
    nNewPos := FTELL(nHandle)
    IF lfreadLn(nHandle, @cBuff)
        lMoved := .T.
        cLine := cBuff
        fseek(nHandle, nNewPos, FS_SET)
    ELSE
        lMoved := .F.
        fseek(nHandle, nSavePos, FS_SET)
    ENDIF

RETURN lMoved



/*

Original routine with bug at EOF

*/

// Attempt to move to the previous line in the file
// Return .T. if successful, .F. otherwise

/*
FUNCTION lGoPrevLn

LOCAL nOrigPos := FTELL(nHandle), nMaxRead, nNewPos, ;
      lMoved, cBuff, nWhereCrLf, nPrev

    IF nOrigPos == 0
        lMoved := .F.
    ELSE
        lMoved := .T.
        IF nOrigPos != nFileSize
            //  Skip over preceeding CR / LF
            fseek(nHandle, -2, FS_RELATIVE)
        ENDIF
        nMaxRead := MIN(MAX_LINE_LEN, FTELL(nHandle))

        cBuff := space(nMaxRead)
        nNewPos := fseek(nHandle, -nMaxRead, FS_RELATIVE)
        fread(nHandle, @cBuff, nMaxRead)
        nWhereCrLf := rat(CRLF, cBuff)
        IF nWhereCrLf == 0
            nPrev := nNewPos
            cLine = cBuff
        ELSE
            nPrev := nNewPos + nWhereCrLf + 1
            cline := substr(cBuff, nWhereCrLf + 2)
        ENDIF

        fseek(nHandle, nPrev, FS_SET)
    ENDIF

RETURN lMoved
*/


/*
   Fixed code
*/

FUNCTION lGoPrevLn

LOCAL nOrigPos := FTELL(nHandle), nMaxRead, nNewPos, ;
      lMoved, cBuff, nWhereCrLf, nPrev, cTemp

    IF nOrigPos == 0
        lMoved := .F.
    ELSE
        lMoved := .T.
        //  Check preceeding 2 chars for CR / LF
        fseek(nHandle, -2, FS_RELATIVE)
        cTemp := space(2)
        fread(nHandle, @cTemp, 2)
        IF cTemp == CRLF
            fseek(nHandle, -2, FS_RELATIVE)
        ENDIF
        nMaxRead := MIN(MAX_LINE_LEN, FTELL(nHandle))

        cBuff := space(nMaxRead)
        nNewPos := fseek(nHandle, -nMaxRead, FS_RELATIVE)
        fread(nHandle, @cBuff, nMaxRead)
        nWhereCrLf := rat(CRLF, cBuff)
        IF nWhereCrLf == 0
            nPrev := nNewPos
            cLine = cBuff
        ELSE
            nPrev := nNewPos + nWhereCrLf + 1
            cline := substr(cBuff, nWhereCrLf + 2)
        ENDIF

        fseek(nHandle, nPrev, FS_SET)
    ENDIF

RETURN lMoved


FUNCTION stdmeth(nKey, toThis)

LOCAL lKeyHandled := .T.

    DO CASE
        CASE nKey == K_DOWN;       toThis:down()
        CASE nKey == K_UP;         toThis:up()
        CASE nKey == K_PGDN;       toThis:pagedown()
        CASE nKey == K_PGUP;       toThis:pageup()
        CASE nKey == K_CTRL_PGUP;  toThis:gotop()
        CASE nKey == K_CTRL_PGDN;  toThis:gobottom()
        CASE nKey == K_RIGHT;      toThis:right()
        CASE nKey == K_LEFT;       toThis:left()
        CASE nKey == K_HOME;       toThis:home()
        CASE nKey == K_END;        toThis:end()
        CASE nKey == K_CTRL_LEFT;  toThis:panleft()
        CASE nKey == K_CTRL_RIGHT; toThis:panright()
        CASE nKey == K_CTRL_HOME;  toThis:panhome()
        CASE nKey == K_CTRL_END;   toThis:panend()
        OTHERWISE;                 lKeyHandled := .F.
    ENDCASE

RETURN lKeyHandled

