* PROCEDURE write
	*
	* Read in any file, create a .prg that uses low-level 
	* funcs to re-create the original.
	*
	* Tested with Clipper Summer '87 and FoxPro LAN 1.02.
	* The only significant syntactical difference is FREAD().
	*
	PARAMETERS inputfile, outputprg, clonefile, make_files, name_seed
	
	******************************************************
	*
	*              TYPE     DEFAULT            DESCRIPTION               	
	* ----------------------------------------------------
	* inputfile  = <exp_C>  NONE - must pass   input file to be 
	*                                          processed.
	*
	* outputprg  = <exp_C>  inputfile, but w/  output file created 
	*                       '.prg' extension   by WRITE.
	*
	* clonefile  = <exp_C>  inputfile          file name that
	*                                          outputprg will 
	*                                          create when run.
	*
	* make_files = <exp_L>  .F.                whether or not to 
	*                                          store procedures as
	*                                          individual disk files.
	*
	* name_seed  = <exp_C>  inputfile, but     name of individual 
	*                       w/o extension      procedure files.
	*                       (also no path if
	*                       make_files = .F.
  *
  *****************************************************
  
	PUBLIC clipper

  PRIVATE xtalk, xstat, xcolo
  xtalk = SET('talk')
	SET TALK OFF
	xcolo = SET('colo')
  SET COLOR TO bg/n
	xstat = SET('status')
	SET STAT OFF
	
	SET CURSOR OFF

	PRIVATE pcount, wildcard, num_matches, j, init_ok, cr_lf
	pcount      =  IIF(clipper,PCOUNT(),PARAMETERS())
	wildcard    = .F.  && if we have a wildcard
	num_matches =  0   && how many files match wildcard filespec
	j           =  0   && 'for' loop counter
	init_ok     = .F.  && If we find a file to work on
	cr_lf       = CHR(13)+CHR(10) 
	
	*
	* No arguments, bring up help screen and exit
	*
	IF pcount = 0
		DO syntax
		DO close_up
		RETURN ""
	ENDIF
	
	**************************************
	*** Fill in the missing parameters ***
	**************************************
	
	*
	* In Clipper, not_passed results in TYPE = "U", _no_ value
	* In FoxPro,  not_passed results in TYPE = "L", value = .F.
	*
	IF TYPE('outputprg')<>"C"   && Param #2
		outputprg = ""
	ENDIF

	IF TYPE('clonefile')<>"C"   && Param #3
		clonefile = ""
	ENDIF
	
	*
	* Ugh - different dialects can be a pain.
	*
	IF TYPE('make_files')<>"L"  && Param #4
		IF clipper     && Always type="C" from DOS command line
			IF TYPE('make_files') = "U"
				make_files = ""
			ENDIF	
			make_files = IIF(UPPER(make_files)=".T.",.T.,.F.)
		ELSE
			make_files = .F.
		ENDIF
	ENDIF
	
	***************************************************
	*** Look for a wildcard in the initial argument ***
	***************************************************
	
	IF ("*" $ inputfile) .OR. ("?" $ inputfile)
		* Got one!
		wildcard = .T.
		IF pcount > 1 .AND. (!EMPTY(outputprg) .OR. !EMPTY(clonefile))
			@0,0 CLEAR
			ret_val = promptyn("WildCard detected - additional parameters " + ;
			 "will be ignored:","OK","Cancel")
			IF EMPTY(ret_val) .OR. ret_val=="Cancel"
				DO close_up
				RETURN ""
			ENDIF
			outputprg = ""  && So we catch them later on
			clonefile = ""
		ENDIF
		IF clipper
			num_matches = adir(inputfile)
			DECLARE names[num_matches]
			null=adir(inputfile,names)
		ELSE
			*
			* Uses my 'f_adir' UDF, will create the array
			* of correct size automatically for us.
			*
			num_matches = f_adir(inputfile,"names")
		ENDIF
	
		******************************
		*** Run the main procedure ***
		******************************
		
		IF num_matches > 0
			FOR j = 1 TO num_matches
				DO main WITH names[j], "", ""
			NEXT
		ELSE
			DO init_err WITH "No matching files were found"
		ENDIF
	ELSE
		DO main WITH inputfile, outputprg, clonefile
	ENDIF  && ("*" $ inputfile) .OR. ("?" $ inputfile)

	DO close_up
		
	RETURN ""
********
PROCEDURE main
	*
	* Here's the main procedure
	*
	PARAMETERS inputfile, outputprg, clonefile
	
  SET COLOR TO bg/n	

	PRIVATE i, counter, inbuff, outbuff, addbuff, handle, buff_len, ;
	 prghandle, temphandle, tempfile, file_len, rep_flag, null, ;
	 StartTime, EndTime, show_row, show_col, elapsed, seed_path
	 
	STORE 0 TO i, counter, handle, prghandle, temphandle, buff_len, ;
	 file_len, null, StartTime, EndTime, show_row, show_col, elapsed
	STORE "" TO inbuff, outbuff, addbuff, tempfile, seed_path
	STORE .F. TO rep_flag       && Tracks whether or not we've used REPL()
	
	*
	* Open all the files, get out if errors
	*
	init_ok = init()
	IF !init_ok
		RETURN
	ENDIF

	*
	* Clipper Summer '87 chokes on long lines that FP can handle. 
	*
	* Since we've made the decision to write one buffer out
	* as one line, use a smaller buffer.  FP runs w/ either size.
	*
	* Buffer size doesn't seem critical to performance, as long as
	* it's not extreme.  25 & 100 yield about the same speed in FP.
	*
	* NB: MOD(10,000,buff_len) MUST EQUAL 0!  This is because we
	* create a new output file/procedure every 10k of input file.
	* For small files, it won't matter, but otherwise it would 
	* involve too many other things everytime this number changes.
	*
	IF clipper
		STORE  25 TO buff_len && Maximum Line length <=  256
	ELSE
		STORE 100 TO buff_len && Maximum Line length <= 1000
	ENDIF
	
	*
	* Error checking for those of you trying out different buffer lengths.
	*
	IF MOD(10000,buff_len) <> 0
		DO init_err WITH "MOD(10,000,buff_len) MUST EQUAL 0"
		DO close_up
		RETURN ""
	ENDIF

	*
	* Our display grid to keep us amused while the work is going on
	*
	show_row = 11
	show_col = 22
	DO show WITH show_row, show_col

	IF make_files
		@show_row+1, show_col+16 SAY PADC("PROC: 0",9)
	ENDIF

	StartTime = seconds()
	
	@show_row+3, show_col+ 6 SAY TRAN(file_len,"9,999,999")
	
	SET COLO TO w+/n
	@24,0 SAY PADC("Processing",WCOL())
	SET COLO TO w+/b

	DO WHILE !FEOF(handle)  && Could use 'counter < file_len'
		*
		* Read from input file
		* Note difference in syntax between CL & FP re: Fread()
		*
		IF clipper
			inbuff   = SPACE(buff_len)
			buff_len = FREAD(handle,@inbuff,buff_len)
		ELSE
			inbuff   = FREAD(handle,buff_len)
			buff_len = LEN(inbuff)
		ENDIF
	
		outbuff  = ""
		rep_flag = .F.
		counter  = counter + buff_len
		
		@show_row+5, show_col+ 6 SAY TRAN(counter,"9,999,999")
		@show_row+3, show_col+26 SAY TRAN(FSEEK(prghandle,0,1),"9,999,999")
		
		elapsed = seconds()-StartTime
		
		@show_row+7, show_col+ 6 SAY TRAN(INT(elapsed/60),"99,999") + ;
		 IIF(MOD(elapsed,60) = 0, ":00", ;
  	 ":" + PADL(LTRIM(TRAN(INT(MOD(elapsed,60)),"99")),2,"0"))

		*******************************************************
		*** Here's the code that does the actual dirty work ***
		*******************************************************
		
		*
		* Read the first char in the buffer
		*
		i = 1
		*
		* If it's alpha
		*
		IF BETWEEN(ASC(SUBSTR(inbuff,i,1)),32,126) .AND. ;
		 !(STR(ASC(SUBSTR(inbuff,i,1)),2) $ ["34"])
			*
			* If the second one is the same as the first, use REPL
			*
	 		IF SUBSTR(inbuff,i+1,1) = SUBSTR(inbuff,i,1)
				rep_flag = .T.
	 			outbuff = [REPL("] + SUBSTR(inbuff,i,1) + ["] + ;
	 			  "," + LTRIM(STR(i+1)) + ")+"
	 			* Now that we've got a 'repl', let's see if they're are more:
	 			i = 2
	 			DO WHILE i <buff_len
		 			IF SUBSTR(inbuff,i+1,1) = SUBSTR(inbuff,i,1)
		 				outbuff = STUFF(outbuff,10,LEN(LTRIM(STR(i))),LTRIM(STR(i+1)))
		 			ELSE
		 				EXIT
		 			ENDIF
	 				i = i + 1
		 		ENDDO
			ELSE
				outbuff = ["] + SUBSTR(inbuff,i,1)
			ENDIF
		ELSE
			*
			* If the second one is the same as the first, use REPL
			*
	 		IF SUBSTR(inbuff,i+1,1) = SUBSTR(inbuff,i,1)
	 			outbuff = "REPL(CHR(" + LTRIM(STR(ASC(SUBSTR(inbuff,i,1)))) + ;
	 			 ")," + LTRIM(STR(i+1)) + ")+"
	 			* Now that we've got a 'repl', let's see if they're are more:
	 			i = 2
	 			DO WHILE i <buff_len
		 			IF SUBSTR(inbuff,i+1,1) = SUBSTR(inbuff,i,1)
		 				outbuff = STUFF(outbuff,;
		 				 12+LEN(LTRIM(STR(ASC(SUBSTR(inbuff,i,1))))),;
		 				 LEN(LTRIM(STR(i))),LTRIM(STR(i+1)))
		 			ELSE
		 				EXIT
		 			ENDIF
	 				i = i + 1
		 		ENDDO
			ELSE
				outbuff = "CHR(" + LTRIM(STR(ASC(SUBSTR(inbuff,i,1)))) + ")+"
			ENDIF
		ENDIF

		*
		* Read the rest of the characters in the buffer.
		* (if entire line is REPL(), loop is not entered)
		*
		i =i + 1
		DO WHILE i <= buff_len
			*
			* If it's alpha
			*
			IF (BETWEEN(ASC(SUBSTR(inbuff,i,1)),32,126) .AND. ;
			 !(STR(ASC(SUBSTR(inbuff,i,1)),2) $ ["34"]))
				* Was the previous one alpha?
				IF BETWEEN(ASC(SUBSTR(inbuff,i-1,1)),32,126) .AND. ;
				 !(STR(ASC(SUBSTR(inbuff,i-1,1)),2) $ ["34"])  .AND. (!rep_flag)
					*
					* 2 alpha's in a row
					*
					* If the second one is the same as the first, use REPL
					*
			 		IF i<buff_len .AND. SUBSTR(inbuff,i+1,1) = SUBSTR(inbuff,i,1)
		 				num_reps = 2
	 					outbufflen = LEN(outbuff)
						rep_flag = .T.
	 					outbuff = outbuff + ["+REPL("] + SUBSTR(inbuff,i,1) + ["] + ;
	 					  "," + LTRIM(STR(num_reps)) + ")+"
			 			* Now that we've got a 'repl', let's see if they're are more:
	 					i = i+1
	 					DO WHILE i <buff_len
				 			IF SUBSTR(inbuff,i+1,1) = SUBSTR(inbuff,i,1)
				 				outbuff = STUFF(outbuff,outbufflen+12,;
		 						 LEN(LTRIM(STR(num_reps))),LTRIM(STR(num_reps+1)))
		 						num_reps = num_reps + 1
				 			ELSE
				 				EXIT
		 					ENDIF
			 				i = i + 1
				 		ENDDO
					ELSE
						rep_flag = .F.
						outbuff = outbuff + SUBSTR(inbuff,i,1) + IIF(i=buff_len,["],"")
					ENDIF
				ELSE
					*
					* CHR-type followed by an alpha
					*
					* If the second one is the same as the first, use REPL
					*
			 		IF i<buff_len .AND. SUBSTR(inbuff,i+1,1) = SUBSTR(inbuff,i,1)
		 				num_reps = 2
	 					outbufflen = LEN(outbuff)
						rep_flag = .T.
	 					outbuff = outbuff + [REPL("] + SUBSTR(inbuff,i,1) + ["] + ;
	 					  "," + LTRIM(STR(num_reps)) + ")+"
			 			* Now that we've got a 'repl', let's see if they're are more:
	 					i = i+1
	 					DO WHILE i <buff_len
				 			IF SUBSTR(inbuff,i+1,1) = SUBSTR(inbuff,i,1)
				 				outbuff = STUFF(outbuff,outbufflen+10,;
		 						 LEN(LTRIM(STR(num_reps))),LTRIM(STR(num_reps+1)))
		 						num_reps = num_reps + 1
				 			ELSE
				 				EXIT
		 					ENDIF
			 				i = i + 1
				 		ENDDO
					ELSE
						rep_flag = .F.
						outbuff = outbuff + ["] + SUBSTR(inbuff,i,1) + ;
						 IIF(i=buff_len,["],"")
					ENDIF
				ENDIF
			ELSE
				*
				* Not alpha, but CHR() type
				*
				* Was the previous one alpha?
				IF BETWEEN(ASC(SUBSTR(inbuff,i-1,1)),32,126) .AND. ;
				 !(STR(ASC(SUBSTR(inbuff,i-1,1)),2) $ ["34"]) .AND. !rep_flag
				 	*
					* Alpha followed by a CHR-type
					*
					* If the second one is the same as the first, use REPL
					*
			 		IF i<buff_len .AND. SUBSTR(inbuff,i+1,1) = SUBSTR(inbuff,i,1)
		 				num_reps = 2
	 					outbufflen = LEN(outbuff)
						rep_flag = .T.
	 					outbuff = outbuff + ["+REPL(CHR(] + ;
	 					 LTRIM(STR(ASC(SUBSTR(inbuff,i,1)))) + [)] + ;
	 					  "," + LTRIM(STR(num_reps)) + ")+"
			 			* Now that we've got a 'repl', let's see if they're are more:
	 					i = i+1
	 					DO WHILE i <buff_len
				 			IF SUBSTR(inbuff,i+1,1) = SUBSTR(inbuff,i,1)
				 				outbuff = STUFF(outbuff,;
				 				outbufflen+14+LEN(LTRIM(STR(ASC(SUBSTR(inbuff,i,1))))),;
				 				 LEN(LTRIM(STR(num_reps))),LTRIM(STR(num_reps+1)))
		 						num_reps = num_reps + 1
				 			ELSE
				 				EXIT
		 					ENDIF
			 				i = i + 1
				 		ENDDO
					ELSE
						rep_flag = .F.
						
						outbuff = outbuff + ["+] + "CHR(" + ;
						 LTRIM(STR(ASC(SUBSTR(inbuff,i,1)))) + ")" + ;
						 IIF(i=buff_len,"","+")
					ENDIF
				ELSE
					*
					* 2 CHR-types
					*
					* If the second one is the same as the first, use REPL
					*
			 		IF i<buff_len .AND. SUBSTR(inbuff,i+1,1) = SUBSTR(inbuff,i,1)
		 				num_reps = 2
	 					outbufflen = LEN(outbuff)
						rep_flag = .T.
	 					outbuff = outbuff + [REPL(CHR(] + ;
	 					 LTRIM(STR(ASC(SUBSTR(inbuff,i,1)))) + [)] + ;
	 					  "," + LTRIM(STR(num_reps)) + ")+"
			 			* Now that we've got a 'repl', let's see if they're are more:
	 					i = i+1
	 					DO WHILE i <buff_len
				 			IF SUBSTR(inbuff,i+1,1) = SUBSTR(inbuff,i,1)
				 				outbuff = STUFF(outbuff,;
				 				outbufflen+12+LEN(LTRIM(STR(ASC(SUBSTR(inbuff,i,1))))),;
				 				 LEN(LTRIM(STR(num_reps))),LTRIM(STR(num_reps+1)))
		 						num_reps = num_reps + 1
				 			ELSE
				 				EXIT
		 					ENDIF
			 				i = i + 1
				 		ENDDO
					ELSE
						rep_flag = .F.

						outbuff = outbuff + "CHR(" + ;
						 LTRIM(STR(ASC(SUBSTR(inbuff,i,1)))) + ")" + ;
						 IIF(i=buff_len,"","+")
					ENDIF
				ENDIF
			ENDIF	&& Is it alpha
			i = i + 1
		ENDDO   && WHILE i <= buff_len

		**************************************************************
		*** We have an entire line.  Now finish and write to disk. ***
		**************************************************************
		*
		* If the entire line is a REPL,
		* we have a "+" to get rid of at the end.
		*
		IF RIGHT(outbuff,1) = "+"
			outbuff = LEFT(outbuff,LEN(outbuff)-1)
		ENDIF

		*
		* Add the closing parenthesis for fwrite()
		*
		outbuff = outbuff + ")" + cr_lf
		
		*
		* If we need it, start a new procedure
		*
		IF counter >= 10000 .AND. MOD(counter,10000) = 0 .AND. !FEOF(handle)
			*
			* Make sure that name_seed is not too long a string
			*
			* FP will allow us to use LEFT(name_seed,7) even if 
			* LEN(name_seed) < 7.  If CL will allow this, too, 
			* then this loop _could_ be eliminated.
			*
			DO WHILE LEN(name_seed) > (8-LEN(LTRIM(STR(counter/10000,10))))
				name_seed = LEFT(name_seed,LEN(name_seed)-1)
			ENDDO
			
			IF make_files
				* Write outbuff to first file
				null=FWRITE(temphandle,outbuff)
				
				* Close first and open second
				null=FCLOSE(temphandle)
				temphandle=FCREATE(seed_path+name_seed+;
				 LTRIM(STR(counter/10000,10))+".PRG")
				
				* Show the new procedure number to keep us amused
				@show_row+1, show_col+16 SAY PADC("PROC: " + ;
				 LTRIM(STR(counter/10000,10)),9)
				outbuff = "* "
			ENDIF

			* Add PROC statement (automatically commented out if make_files = .T.)
			outbuff = outbuff + ;
			 "PROCEDURE "+name_seed+LTRIM(STR(counter/10000,10)) + cr_lf 
	
			* Add a DO statement to the output .prg
			null=FPUTS(prghandle,"DO "+name_seed+;
			 LTRIM(STR(counter/10000,10)))
		ENDIF

		* Add the start of a new line to 'outbuff'
		outbuff=outbuff+IIF(!FEOF(handle),;
		 "null=FWRITE(handle,", "null=FCLOSE(handle)") 
			
		* Write 'outbuff'
		null=FWRITE(temphandle,outbuff)
		@show_row+3, show_col+16 SAY TRAN(FSEEK(temphandle,0,1),"9,999,999")

	ENDDO   && WHILE !FEOF(handle)
	
	***********************************************
	*** We're through processing the input file ***
	***********************************************
	
	IF make_files
		DO closmany
	ELSE
		DO closfile
	ENDIF

	RETURN
********
FUNCTION init
	*
	* Called from main(), opens all files,
	* supplies missing params, etc.
	*
	* Open the input file
	*
	* Add a .dbf extension if no extension is supplied
	IF !("." $ RIGHT(inputfile,4))
		inputfile = inputfile + ".dbf"
	ENDIF

	handle = FOPEN(inputfile)
	IF handle = -1
		DO init_err WITH "Could not "+IIF(!FILE(inputfile),"find ","open ")+inputfile
		RETURN .F.
	ENDIF
	
	file_len = FSEEK(handle,0,2)    && Get length of file
	null     = FSEEK(handle,0,0)    && Put the pointer back at FBOF()

	*
	* Name the output file, if necessary, and open
	*
	IF pcount < 2 .OR. EMPTY(outputprg)
		* Get rid of extension, if any
		IF AT(".",inputfile) <> 0  && always true? leave for safety
			outputprg = LEFT(inputfile,AT(".",inputfile)-1)
		ENDIF
	ENDIF
	
	* Add a .prg extension if no extension is supplied
	IF !("." $ RIGHT(outputprg,4))
		outputprg = outputprg + ".prg"
	ENDIF
	
	IF FILE(outputprg) .AND. !wildcard
		@0,0 CLEAR
		ret_val = promptyn(outputprg + " exists. Overwrite?")
		IF ret_val = "No" .OR. EMPTY(ret_val)
			null=FCLOSE(handle)
			RETURN .F.
		ENDIF
	ENDIF

	prghandle = FCREATE(outputprg)
	IF prghandle = -1
		DO init_err WITH "Could not create " + outputprg
		null=FCLOSE(handle)
		RETURN .F.
	ENDIF

	IF TYPE('name_seed')<>"C"
		name_seed = outputprg
	ELSE
		name_seed = IIF(EMPTY(name_seed),outputprg,name_seed)
	ENDIF
	*
	* Get the path in name_seed, if there is one
	*
	IF SUBSTR(name_seed,2,1) == ":"
		seed_path = LEFT(name_seed,2)
		name_seed = RIGHT(name_seed,LEN(name_seed)-2)
	ENDIF
	*
	* Will be "" if not found
	seed_path = seed_path + LEFT(name_seed,RAT("\",name_seed))
	*
	* What's left is the file name
	*
	name_seed = RIGHT(name_seed,LEN(name_seed)-RAT("\",name_seed))
	*
	* Throw away the extension if any
	*
	DO WHILE AT(".",name_seed) <> 0
		name_seed = LEFT(name_seed,LEN(name_seed)-1)
	ENDDO
	*
	* Now make sure it's short enough
	*
	name_seed = LEFT(name_seed,7)

	IF make_files
		*
		* File "A0.PRG" or whatever
		* These are overwritten if they exists w/o prompting the user
		*
		tempfile = seed_path+name_seed + "0.PRG"
	ELSE
		*
		* Get a unique temp_file name and open it
		*
		tempfile = SYS(3)
	ENDIF

	temphandle = FCREATE((tempfile))
	IF temphandle = -1
		DO init_err WITH "Could not create " + ;
		 IIF(make_files,"first .prg","temporary")+" file"
		null=FCLOSE(handle)
		null=FCLOSE(prghandle)
		RETURN .F.
	ENDIF
	
	IF pcount < 3 .OR. EMPTY(clonefile)
		clonefile = inputfile
	ENDIF
	
	* Add a .dbf extension if no extension is supplied
	IF !("." $ RIGHT(clonefile,4))
		clonefile = clonefile + ".dbf"
	ENDIF

	*
	* Get the short name of the output file for writing
	* it as a comment at bof(outputprg).  Use 'null', since
	* we don't need this var after this.
	*
	* Get rid of drive
	*
	null = outputprg
	IF SUBSTR(outputprg,2,1) == ":"
		null = RIGHT(outputprg,LEN(outputprg)-2)
	ENDIF
	*
	* Get rid of path
	*
	null = RIGHT(null,LEN(null)-RAT("\",null))
	*
	* Get rid of extension
	*
	DO WHILE AT(".",null) <> 0
		null = LEFT(null,LEN(null)-1)
	ENDDO

	*
	* Write the beginning of the .prg
	*
	*** If additional start-up code in the output .prg
	*** is desired, insert it at the appropriate place
	*** within the next group of 'null=FWRITE' lines.
	***
	*** For instance, to check if the file exists and
	*** stop execution if it does:
	***
	***   null=FPUTS(prghandle,'IF FILE("' + clonefile '")' )
	***   null=FPUTS(prghandle,CHR(9) + 'WAIT "File already exists."')
  ***   null=FPUTS(prghandle,CHR(9) + 'RETURN')		
	***   null=FPUTS(prghandle,'ENDIF')
	***
	*** could be inserted before the "DO" line.
	*
	null=FPUTS(prghandle,'* PROCEDURE ' + null)
	null=FPUTS(prghandle,;
	 "* Source code generated by  W R I T E  (c) 1991, Friday's Computer, NYC")
	null=FPUTS(prghandle,'handle=FCREATE("' + clonefile + '")')
	null=FPUTS(prghandle,"IF handle = -1")
	null=FPUTS(prghandle,;
	 CHR(9)+'WAIT "Could not create '+ clonefile+'. Press any key."')
	null=FPUTS(prghandle,CHR(9)+"RETURN")
	null=FPUTS(prghandle,"ENDIF")
	null=FPUTS(prghandle,"DO "+name_seed+"0")

	*
	* Write the beginning of the 'writing' code
	* to the temporary file
	*
	IF make_files
		null=FWRITE(temphandle,"* ")  && Comment-out PROC statement
	ENDIF
	
	null=FWRITE(temphandle,"PROCEDURE "+name_seed+"0" + cr_lf + ;
	 "null=FWRITE(handle,")

	RETURN .T.
********
PROCEDURE init_err
	* Called from init() and top proc
	PARAMETERS mess
	IF clipper
		? mess
	ELSE
		WAIT mess + ".  Press any key."
	ENDIF
	RETURN
********
PROCEDURE closmany
	*
	* Called only if make_files = .T.
	*
	SET COLOR TO w+/n
	
	@24, 0 SAY PADC(;
	 "Closing input and output files.  Please wait.",;
	 WCOL())

	null=FCLOSE(temphandle)  && The last of the separate .prgs
	null=FCLOSE(handle)
	null=FCLOSE(prghandle)

	@24,0
	SET COLOR TO w+/b
	@show_row+3, show_col+16 SAY SPACE(9)
	RETURN
********
PROCEDURE closfile
	*
	* Called only if make_files = .F.
	*
	* Must copy tempfile into outputprg before closing
	*
	elapsed = seconds()-StartTime
	@show_row+7, show_col+ 6 SAY TRAN(INT(elapsed/60),"99,999") + ;
	 IIF(MOD(elapsed,60) = 0, ":00", ;
   ":" + PADL(LTRIM(TRAN(INT(MOD(elapsed,60)),"99")),2,"0"))
	
	StartTime = seconds()
	*
	* Read the temp file into the output prg file.
	*
	* If counter <10,000, we don't really need the
	* procedure business, so get rid of it.
	*
	IF counter < 10000
		*
		* Move forward in temp file to skip 'PROCEDURE A0' and
		* move back in output file to skip 'DO A0'
		*
		PRIVATE len_proc
		len_proc = LEN(name_seed)+LEN(LTRIM(STR(counter/10000,10)))
		* Whew!
		@show_row+3, show_col+16 SAY ;
		 TRAN(FSEEK(temphandle,0,1)-(12+len_proc),"9,999,999")
		@show_row+3, show_col+26 SAY ;
		 TRAN(FSEEK(prghandle,0,1) -( 5+len_proc),"9,999,999")
		null     = FSEEK(temphandle,(12+len_proc),0)
		file_len = FSEEK(prghandle,-( 5+len_proc),2)
		RELEASE len_proc
	ELSE
		null     = FSEEK(temphandle,0,0)
		file_len = FSEEK(prghandle,0,2)
	ENDIF	
	counter  = 0
	buff_len = 30000  && Occasion weirdness in FP 1.02 w/ 64000 here
	
	*
	* Read in tempfile
	*
	DO WHILE !FEOF(temphandle)
		@show_row+5, show_col+16 SAY TRAN(counter,"9,999,999")
		IF clipper
			inbuff   = SPACE(buff_len)
			null     = FREAD(temphandle,@inbuff,buff_len)
			buff_len = FWRITE(prghandle,TRIM(inbuff))
		ELSE
			null     = FWRITE(prghandle,FREAD(temphandle,buff_len))
		ENDIF

		counter  = counter + buff_len

		@show_row+3, show_col+26 SAY TRAN(file_len+counter,"9,999,999")
		elapsed = seconds()-StartTime
		@show_row+7, show_col+ 16 SAY TRAN(INT(elapsed/60),"99,999") + ;
		 IIF(MOD(elapsed,60) = 0, ":00", ;
  	 ":" + PADL(LTRIM(TRAN(INT(MOD(elapsed,60)),"99")),2,"0"))
	ENDDO

	*
	* For FP, we need to accurately reflect the partial buffer
	* we read on the last pass since we never got buff_len.
	*
	@show_row+3, show_col+26 SAY TRAN(FSEEK(prghandle,0,2),"9,999,999")
	
	@show_row+5, show_col+16 SAY TRAN(FSEEK(temphandle,0,1),"9,999,999")
	elapsed = seconds()-StartTime
	@show_row+7, show_col+ 16 SAY TRAN(INT(elapsed/60),"99,999") + ;
	 IIF(MOD(elapsed,60) = 0, ":00", ;
   ":" + PADL(LTRIM(TRAN(INT(MOD(elapsed,60)),"99")),2,"0"))
	
	*
	* Close 'em up
	*
	SET COLOR TO w+/n
	
	@24, 0 SAY PADC(;
	 "Erasing temporary file, closing input and output files.  Please wait.",;
	 80)

	null=FCLOSE(temphandle)
	ERASE (tempfile)		
	null=FCLOSE(handle)
	null=FCLOSE(prghandle)

	@24,0
	SET COLOR TO w+/b
	@show_row+3, show_col+16 SAY "(erased) "
	RETURN
********
PROCEDURE close_up
	*
	* Called from top proc when everything is done
	*
	IF init_ok  && Don't do if we never really got started
		*
		* Let's ring the bell and go home
		*
		* A=444 sounds brighter - used by many European orchestras
		*
		* 12th root of 2 is half-step multiplier
		*
		null=our_tone(222 * (1.059464^ 7),4)
		null=our_tone(222 * (1.059464^10),4)
		null=our_tone(222 * (1.059464^15),4)
		SET COLOR TO w+/n
		@24, 0 SAY PADC("FINISHED.  " + ;
		 IIF(num_matches>0,LTRIM(STR(num_matches)) + ;
		 " file(s) prepared.  ","") + ;
		 "Press any key.", 80)
		key = INKEY(0)
	ENDIF
	
	RELEASE clipper
	SET CURSOR ON
	SET COLO TO &xcolo
	SET TALK &xtalk
	SET STAT &xstat
	RETURN
********
FUNCTION our_tone
	* For FoxPro, but both, really
	PARAMETERS frequency, duration
	IF clipper
		null = TONE(frequency, duration)  && In Extend.lib
	ELSE
		SET BELL TO frequency, duration
		??CHR(7)
		SET BELL TO
	ENDIF
	RETURN ""
********
FUNCTION promptyn
	*
	* 
	*
	PARAMETERS mess, choice_1, choice_2, top, left

	*
	* mess      = <exp_C>, message to display in the box
	* choice_1  = <exp_C>, first of the two choices
	* choice_2  = <exp_C>, second of the two choices
	* top       = <exp_N>, top row of the window
	* left      = <exp_N>, left column of the window
	
	**************************
	* Local Memory Variables * 
	**************************
	
	PRIVATE promptvar, ret_val, pcount

	pcount = IIF(clipper,PCOUNT(),PARAMETERS())

	* Initialize the menu variable
	promptvar =1

	* What we return to the calling .prg
	ret_val = SPACE(5)
	
	**************************************************
	* Fill in some parameters if they weren't passed *
	**************************************************

	IF pcount < 1
		mess = "Please Choose:"
	ENDIF
	
	IF pcount < 3 .OR. ;
	 IIF(!clipper,(choice_1 == "" .AND. choice_2 == ""),.F.)
		choice_1 = "Yes"
		choice_2 = "No"
	ENDIF

	* Pad everything with spaces to make it look nicer
	choice_1 = " " + choice_1 + " "
	choice_2 = " " + choice_2 + " "

	**************************************
	* Calculate window size and location *
	**************************************
	
	* Figure out the window size
	winwidth = MIN(LEN(mess) + LEN(choice_1) + LEN(choice_2) + 5, WCOL())
	
	* Now the location, more parameters if necessary
	IF pcount < 4 
		top = 99
	ENDIF
	
	* If not passed _or_ passed 99
	IF top = 99
		top = WROW() / 2 - 2
	ENDIF
	DO WHILE top + 1 > WROW()
		top = top - 1
	ENDDO
	bottom = top + 4
		
	* If not passed _or_ passed 99
	IF pcount < 5
		left = 99
	ENDIF
	IF left = 99
		left = (WCOL()-winwidth)/2
	ENDIF
	DO WHILE left+winwidth > WCOL()
		left = left - 1
	ENDDO
	right = left + winwidth
				
	*****************************************************************
	* Define and activate the window, put the choices on the screen *
	*****************************************************************

  solid2 = " "
  @ top,left,bottom,right BOX solid2
	@bottom-2,left+2 SAY mess

	@ROW(),COL()+1 PROMPT choice_1 MESSAGE " "
	@ROW(),COL()+1 PROMPT choice_2 MESSAGE " "
	MENU TO promptvar
	
	***************************
	* Clean house and go home *
	***************************
	
  @ top,left CLEAR TO bottom,right
	
	IF EMPTY(promptvar)  && They pressed <escape>
	  ret_val = ""
	ELSE
		ret_val = IIF(promptvar=1,ALLTRIM(choice_1),ALLTRIM(choice_2))
	ENDIF

	RETURN ret_val
********
PROCEDURE show
	*
	* Called from main() to put up our display grid.
	*	
	PARAMETERS row, col
	PRIVATE double, width, top
  double = "ͻȺ "
	width = 14 + MAX(MAX(LEN(inputfile),LEN(outputprg)),LEN(clonefile))
	top = row - 7

  @0,0 CLEAR
  @1,0,23,79 BOX "" + CHR(177)      && Solid box with background

  SET COLOR TO w+/n
  @1,18 SAY " WRITE 1.2, (c) 1991 Friday's Computer, NYC "

  SET COLOR TO w+/b  && This is the color we'll kepp

	@top,((80-width)/2)-2, top + 4,((80+width)/2)+2 BOX double
	@top  +1, (80-width)/2 SAY "INPUT FILE   : " + inputfile
	@ROW()+1, (80-width)/2 SAY "OUTPUT FILE  : " + outputprg
	@ROW()+1, (80-width)/2 SAY "WILL RECREATE: " + clonefile
	?
	@row    , col SAY "ͻ"
	@ROW()+1, col SAY "  INPUT  TEMPORARY  OUTPUT "
	@ROW()+1, col SAY "Ķ"
	@ROW()+1, col SAY "SIZE                           "
	@ROW()+1, col SAY "Ķ"
	@ROW()+1, col SAY "DONE                           "
	@ROW()+1, col SAY "Ĵ         "
	@ROW()+1, col SAY "TIME                           "
	@ROW()+1, col SAY "ͼ"
	
	RETURN
********
PROCEDURE syntax
	*
	* Called only if no command-line parameters are passed.
	*
	SET CURSOR ON
	TEXT
  WRITE <input_file> [<output_file>] [<recreated_file>] 
    [<separate_files>] [<seed>]
  Version 1.2, (c) 1991 Friday's Computer, NYC

    WRITE reads <input_file> and produces source code that, when run,
    will recreate it.

    You may specify a name for the <output_file>; otherwise, WRITE
    will use <input_file>, but ending with .PRG.

    The file to be recreated may be given a different name by specifying
    <recreated_file>.  WRITE's default is <input_file>.

    The wildcards "*" and "?" are permissible in <input_file>.  If they are
    are used, then <output_file> and <recreated_file> are automatically
    determined by WRITE, and any user-specified <output_file> or 
    <recreated_file> is ignored.
    
    <separate_files> is a logical-type parameter that, if True, will break
    large amounts of source code into separate disk files.  <seed> is used
    to begin the name of either these disk files or, if <separate_files>
    is False, the procedures within <outputprg>.
	ENDTEXT
	?
	IF !clipper
		WAIT
	ENDIF
	RETURN
********

*****************************************************
*** The remaining functions are to compensate for ***
*** differences between FoxPro and Clipper.       *** 
*****************************************************

********
FUNCTION sys
	* For Clipper
	PARAMETERS number1, number2
	PRIVATE ret_val
	DO CASE
	CASE number1 = 3        && Random file name
		ret_val = RIGHT(DTOS(DATE()),2) + substr(time(),1,2) + ;
		 substr(time(),4,2) + substr(time(),7,2) 
	CASE number1 = 2000     && Directory services, never called
		ret_val = ""
	ENDCASE
	RETURN ret_val
********
FUNCTION between
	* For Clipper
	PARAMETERS item, min, max
	RETURN IIF(item >= min .AND. item <= max, .T., .F.)
********
FUNCTION fputs
	* For Clipper
	PARAMETERS handle, string
	null=FWRITE(handle,string+cr_lf)
	RETURN ""
********
FUNCTION feof
	* For Clipper
	PARAMETERS fhandle
	PRIVATE curr_pos, eof
	curr_pos = FSEEK(fhandle,0,1)        && 0 from current position
	eof      = FSEEK(fhandle,0,2)        && 0 from end
	null     = FSEEK(fhandle,curr_pos,0) && Back to original position
	RETURN IIF(curr_pos = eof, .T., .F.)
********
FUNCTION padc
	* For Clipper
	PARAMETERS string, width  && Could add 'char' like padl
	RETURN SPACE((width-LEN(string))/2)+string+SPACE((width-LEN(string))/2)
********
FUNCTION padl
	* For Clipper
	PARAMETERS string, width, char
	IF TYPE('char') <> "C"
		char = " "
	ENDIF
	RETURN REPL(char,width-LEN(string))+string 
********
FUNCTION parameters
	* For Clipper
	RETURN ""  && Must use PCOUNT() in calling .prg
********
FUNCTION set
	* For Clipper
	PARAMETERS who_cares
	PRIVATE ret_val
	ret_val = ""
	DO CASE
	CASE who_cares = "talk"
		ret_val = "OFF"
	CASE who_cares = "stat"
		ret_val = "OFF"
	CASE who_cares = "colo"
		ret_val = ""
	ENDCASE
	RETURN ret_val
********
FUNCTION wrow
	* For Clipper
	RETURN 25
********
FUNCTION wcol
	* For Clipper
	RETURN 80
********
FUNCTION f_adir
	*
	* Date   : 22 April 89
	* Author : Steve Freides
	* Returns: <exp_N>, the number of matching files
	*
	*	  Notes: Useful, with achoice(), for allowing selection
	*          of files from disk.
	*
	*          Extracted from my ARRAYS.PRG, printed in FoxTALK
	*          December 1990, available in earlier form on CIS.
	*
	PARAMETERS skeleton, a_name
	*
	*  skeleton = <exp_C>, expression to match
	*    a_name = <exp_C>, the name of a Public array to create.
	*                      If "" is passed, no array will be created.
	*
	IF TYPE('a_name') <> "C"
		a_name = ""
	ENDIF
	PRIVATE num, a_len
	num      = 0 
	a_len    = 0

	IF LEN(SYS(2000,skeleton)) > 0
		a_len = 1
	ENDIF
	DO WHILE LEN(SYS(2000,skeleton,1)) > 0
    a_len = a_len + 1
	ENDDO

	IF a_len > 0 .AND. LEN(a_name) > 0
	  RELEASE &a_name
   	PUBLIC &a_name(a_len)
	  num = 1
    &a_name(num) = SYS(2000,skeleton)
   	DO WHILE num < a_len
      num = num + 1
      &a_name(num) = SYS(2000,skeleton,1)
	  ENDDO
	ENDIF    

	RETURN a_len
********        

**************************
**                      **
*     EOF(write.prg)     *
**                      **
**************************
